@@ -1,21 +1,8 @@
-// File_Aac - Info for AAC (Raw) files
-// Copyright (C) 2008-2012 MediaArea.net SARL, Info@MediaArea.net
-//
-// This library is free software: you can redistribute it and/or modify it
-// under the terms of the GNU Library General Public License as published by
-// the Free Software Foundation, either version 2 of the License, or
-// any later version.
-//
-// This library is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-// GNU Library General Public License for more details.
-//
-// You should have received a copy of the GNU Library General Public License
-// along with this library. If not, see <http://www.gnu.org/licenses/>.
-//
-//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
-//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
+/*  Copyright (c) MediaArea.net SARL. All Rights Reserved.
+ *
+ *  Use of this source code is governed by a BSD-style license that can
+ *  be found in the License.html file in the root of the source tree.
+ */
 
 //---------------------------------------------------------------------------
 // Pre-compilation
@@ -52,7 +39,7 @@ extern const char* Aac_audioObjectType(int8u audioObjectType);
 extern const char* Aac_Format_Profile(int8u ID);
 
 //---------------------------------------------------------------------------
-const char* Aac_id_syn_ele[8]=
+static const char* Aac_id_syn_ele[8]=
 {
     "SCE - single_channel_element",
     "CPE - channel_pair_element",
@@ -65,7 +52,7 @@ const char* Aac_id_syn_ele[8]=
 };
 
 //---------------------------------------------------------------------------
-const char* Aac_window_sequence[4]=
+static const char* Aac_window_sequence[4]=
 {
     "ONLY_LONG_SEQUENCE",
     "LONG_START_SEQUENCE",
@@ -146,6 +133,12 @@ void File_Aac::program_config_element()
         Skip_S1(2,                                              "matrix_mixdown_idx");
         Skip_SB(                                                "pseudo_surround_enable");
     TEST_SB_END();
+    bool front1_element_is_cpe=false;
+    if (!num_side_channel_elements && num_back_channel_elements && num_back_channel_elements<3) // Hack: e.g. in case of 5.1, 7.1
+    {
+        num_side_channel_elements=1;
+        num_back_channel_elements--;
+    }
     for (int8u Pos=0; Pos<num_front_channel_elements; Pos++)
     {
         Element_Begin1("front_element");
@@ -156,6 +149,8 @@ void File_Aac::program_config_element()
         {
             Channels_Front+=2;
             Channels+=2;
+            if (Pos==0)
+                front1_element_is_cpe=true;
         }
         else
         {
@@ -229,46 +224,49 @@ void File_Aac::program_config_element()
     Element_End0();
 
     //Filling
-    Ztring Channels_Positions, Channels_Positions2;
+    Ztring Channels_Positions, Channels_Positions2, ChannelLayout;
     switch (Channels_Front)
     {
         case  0 : break;
-        case  1 : Channels_Positions+=__T("Front: C"); break;
-        case  2 : Channels_Positions+=__T("Front: L R"); break;
-        case  3 : Channels_Positions+=__T("Front: L C R"); break;
-        default : Channels_Positions+=__T("Front: "); Channels_Positions+=Ztring::ToZtring(Channels_Front); //Which config?
+        case  1 : Channels_Positions+=__T("Front: C"); ChannelLayout+=__T("C "); break;
+        case  2 : Channels_Positions+=__T("Front: L R"); ChannelLayout+=__T("L R "); break;
+        case  3 : Channels_Positions+=__T("Front: L C R"); ChannelLayout+=num_front_channel_elements==2?(front1_element_is_cpe?__T("L R C "):__T("C L R ")):__T("? ? ? "); break;
+        default : Channels_Positions+=__T("Front: "); Channels_Positions+=Ztring::ToZtring(Channels_Front); ChannelLayout+=__T("? "); //Which config?
     }
     switch (Channels_Side)
     {
         case  0 : break;
-        case  1 : Channels_Positions+=__T(", Side: C"); break;
-        case  2 : Channels_Positions+=__T(", Side: L R"); break;
-        case  3 : Channels_Positions+=__T(", Side: L C R"); break;
-        default : Channels_Positions+=__T(", Side: "); Channels_Positions+=Ztring::ToZtring(Channels_Side); //Which config?
+        case  1 : Channels_Positions+=__T(", Side: C"); ChannelLayout+=__T("Cs "); break;
+        case  2 : Channels_Positions+=__T(", Side: L R"); ChannelLayout+=__T("Ls Rs "); break;
+        case  3 : Channels_Positions+=__T(", Side: L C R"); ChannelLayout+=__T("? ? ? "); break;
+        default : Channels_Positions+=__T(", Side: "); Channels_Positions+=Ztring::ToZtring(Channels_Side); ChannelLayout+=__T("? "); //Which config?
     }
     switch (Channels_Back)
     {
         case  0 : break;
-        case  1 : Channels_Positions+=__T(", Back: C"); break;
-        case  2 : Channels_Positions+=__T(", Back: L R"); break;
-        case  3 : Channels_Positions+=__T(", Back: L C R"); break;
-        default : Channels_Positions+=__T(", Back: "); Channels_Positions+=Ztring::ToZtring(Channels_Back); //Which config?
+        case  1 : Channels_Positions+=__T(", Back: C"); ChannelLayout+=__T("Cs "); break;
+        case  2 : Channels_Positions+=__T(", Back: L R"); ChannelLayout+=__T("Rls Rrs "); break;
+        case  3 : Channels_Positions+=__T(", Back: L C R"); ChannelLayout+=__T("Rls Cs Rrs "); break;
+        default : Channels_Positions+=__T(", Back: "); Channels_Positions+=Ztring::ToZtring(Channels_Back); ChannelLayout+=__T("? "); //Which config?
     }
     switch (Channels_LFE)
     {
         case  0 : break;
-        case  1 : Channels_Positions+=__T(", LFE"); break;
-        default : Channels_Positions+=__T(", LFE= "); Channels_Positions+=Ztring::ToZtring(Channels_LFE); //Which config?
+        case  1 : Channels_Positions+=__T(", LFE"); ChannelLayout+=__T("LFE "); break;
+        default : Channels_Positions+=__T(", LFE= "); Channels_Positions+=Ztring::ToZtring(Channels_LFE); ChannelLayout+=__T("? "); //Which config?
     }
     Channels_Positions2=Ztring::ToZtring(Channels_Front)+__T('/')
                        +Ztring::ToZtring(Channels_Side)+__T('/')
                        +Ztring::ToZtring(Channels_Back)
                        +(Channels_LFE?__T(".1"):__T(""));
+    if (!ChannelLayout.empty())
+        ChannelLayout.resize(ChannelLayout.size()-1);
 
     FILLING_BEGIN();
         //Integrity test
-        if (Aac_sampling_frequency[sampling_frequency_index_Temp]==0 || Channels_Front<Channels_Side || Channels_Side<Channels_Back)
+        if (Aac_sampling_frequency[sampling_frequency_index_Temp]==0 || Channels>24) // TODO: full_2023548870.mp4 is buggy
         {
+            Trusted_IsNot("sampling frequency / channels");
             Skip_BS(Data_BS_Remain(),                               "(Unknown frequency)");
             return;
         }
@@ -287,12 +285,13 @@ void File_Aac::program_config_element()
         Infos["Channel(s)"].From_Number(Channels);
         Infos["ChannelPositions"]=Channels_Positions;
         Infos["ChannelPositions/String2"]=Channels_Positions2;
+        Infos["ChannelLayout"]=ChannelLayout;
 
         if (!Infos["Format_Settings_SBR"].empty())
         {
             Infos["Format_Profile"]=__T("HE-AAC");
             Ztring SamplingRate=Infos["SamplingRate"];
-            Infos["SamplingRate"].From_Number((extension_sampling_frequency_index==(int8u)-1)?(sampling_frequency*2):extension_sampling_frequency, 10);
+            Infos["SamplingRate"].From_Number((extension_sampling_frequency_index==(int8u)-1)?(Frequency_b*2):extension_sampling_frequency, 10);
             if (MediaInfoLib::Config.LegacyStreamDisplay_Get())
             {
                 Infos["Format_Profile"]+=__T(" / LC");
@@ -315,7 +314,7 @@ void File_Aac::program_config_element()
                 Infos["Format_Profile"]+=__T(" / HE-AAC / LC");
                 Infos["Channel(s)"]+=__T(" / ")+Channels+__T(" / ")+Channels;
                 Infos["ChannelPositions"]+=__T(" / ")+ChannelPositions+__T(" / ")+ChannelPositions;
-                Infos["SamplingRate"]=Ztring().From_Number((extension_sampling_frequency_index==(int8u)-1)?(sampling_frequency*2):extension_sampling_frequency, 10)+__T(" / ")+SamplingRate;
+                Infos["SamplingRate"]=Ztring().From_Number((extension_sampling_frequency_index==(int8u)-1)?(Frequency_b*2):extension_sampling_frequency, 10)+__T(" / ")+SamplingRate;
             }
             Infos["Format_Settings_PS"]=__T("Yes (Implicit)");
             Ztring Codec=Retrieve(Stream_Audio, StreamPos_Last, Audio_Codec);
@@ -331,19 +330,19 @@ void File_Aac::program_config_element()
 //---------------------------------------------------------------------------
 void File_Aac::raw_data_block()
 {
-    if (audioObjectType!=2)
-    {
-        Skip_BS(Data_BS_Remain(),                               "Data");
-        return; //We test only AAC LC
-    }
-
-    if (sampling_frequency_index>=16 || Aac_sampling_frequency[sampling_frequency_index]==0)
+    if (sampling_frequency_index>=13)
     {
         Trusted_IsNot("(Problem)");
         Skip_BS(Data_BS_Remain(),                               "(Problem)");
         return;
     }
 
+    if (audioObjectType!=2)
+    {
+        Skip_BS(Data_BS_Remain(),                               "Data");
+        return; //We test only AAC LC
+    }
+
     //Parsing
     Element_Begin1("raw_data_block");
     int8u id_syn_ele=0, id_syn_ele_Previous;
@@ -352,6 +351,12 @@ void File_Aac::raw_data_block()
         Element_Begin0();
         id_syn_ele_Previous=id_syn_ele;
         Get_S1 (3, id_syn_ele,                                  "id_syn_ele"); Param_Info1(Aac_id_syn_ele[id_syn_ele]); Element_Name(Aac_id_syn_ele[id_syn_ele]);
+
+        #if MEDIAINFO_TRACE
+            bool Trace_Activated_Save=Trace_Activated;
+            Trace_Activated=false; //It is too big, disabling trace for now for full AAC parsing
+        #endif //MEDIAINFO_TRACE
+
         switch (id_syn_ele)
         {
             case 0x00 : single_channel_element();           break; //ID_SCE
@@ -364,23 +369,17 @@ void File_Aac::raw_data_block()
             case 0x07 :                                     break; //ID_END
             default   :                                          ; //Can not happen
         }
+
+        #if MEDIAINFO_TRACE
+            Trace_Activated=Trace_Activated_Save;
+        #endif //MEDIAINFO_TRACE
+
         Element_End0();
     }
     while(Element_IsOK() && Data_BS_Remain() && id_syn_ele!=0x07); //ID_END
     if (Element_IsOK() && Data_BS_Remain()%8)
         Skip_S1(Data_BS_Remain()%8,                             "byte_alignment");
     Element_End0();
-
-    if (sampling_frequency)
-    {
-        if (FrameInfo.PTS!=(int64u)-1)
-            Element_Info1(__T("PTS ")+Ztring().Duration_From_Milliseconds(float64_int64s(((float64)FrameInfo.PTS)/1000000)));
-        if (FrameInfo.DTS!=(int64u)-1)
-            Element_Info1(__T("DTS ")+Ztring().Duration_From_Milliseconds(float64_int64s(((float64)FrameInfo.DTS)/1000000)));
-        FrameInfo.DUR=float64_int64s(((float64)frame_length)*1000000000/sampling_frequency);
-        FrameInfo.DTS+=FrameInfo.DUR;
-        FrameInfo.PTS=FrameInfo.DTS;
-    }
 }
 
 //---------------------------------------------------------------------------
@@ -796,7 +795,7 @@ void File_Aac::section_data()
             sect_len+=sect_len_incr;
             sect_start[g][i]=k;
             sect_end[g][i]=k+sect_len;
-            for (int8u sfb=k; sfb<k+sect_len; sfb++)
+            for (int16u sfb=k; sfb<k+sect_len; sfb++)
                 sfb_cb[g][sfb]=sect_cb[g][i];
             k+= sect_len;
             i++;
@@ -1082,6 +1081,7 @@ void File_Aac::extension_payload(size_t End, int8u id_aac)
                                     break;
                         default:    ;
                     }
+                    break;
         case 0 : //EXT_FILL
         default:
             Skip_BS(Data_BS_Remain()-End,                       "other_bits");
@@ -1277,9 +1277,12 @@ void File_Aac::ELDSpecificConfig ()
     }
 
     int8u eldExtType;
-    Get_S1(4,eldExtType,"eldExtType");
-    while (eldExtType != 0/*ELDEXT_TERM*/)
+    for (;;)
     {
+        Get_S1(4,eldExtType,"eldExtType");
+        if (eldExtType == 0/*ELDEXT_TERM*/)
+            break;
+
         int8u eldExtLen,eldExtLenAdd=0;
         int16u eldExtLenAddAdd;
         Get_S1(4,eldExtLen,"eldExtLen");
