@@ -24,6 +24,7 @@
 #if defined(MEDIAINFO_MPEG4V_YES) || defined(MEDIAINFO_MPEG4_YES) || defined(MEDIAINFO_MXF_YES)
 //---------------------------------------------------------------------------
 
+#include "MediaInfo/MediaInfo_Config_MediaInfo.h"
 #if MEDIAINFO_EVENTS
     #include "MediaInfo/MediaInfo_Events_Internal.h"
 #endif //MEDIAINFO_EVENTS
@@ -247,7 +248,7 @@ File_Mpeg4v::File_Mpeg4v()
     IsRawStream=true;
 
     //In
-    Frame_Count_Valid=MediaInfoLib::Config.ParseSpeed_Get()>=0.3?30:2;
+    Frame_Count_Valid=0;
     FrameIsAlwaysComplete=false;
 
     //Temp
@@ -301,6 +302,9 @@ bool File_Mpeg4v::Synched_Test()
 //---------------------------------------------------------------------------
 void File_Mpeg4v::Synched_Init()
 {
+    if (!Frame_Count_Valid)
+        Frame_Count_Valid=Config->ParseSpeed>=0.3?30:2;
+
     //Count of a Packets
     IVOP_Count=0;
     PVOP_Count=0;
@@ -664,7 +668,7 @@ bool File_Mpeg4v::Demux_UnpacketizeContainer_Test()
             Demux_Offset++;
         }
 
-        if (Demux_Offset+4>Buffer_Size && File_Offset+Buffer_Size!=File_Size)
+        if (Demux_Offset+4>Buffer_Size && !Config->IsFinishing)
             return false; //No complete frame
 
         if (!Status[IsAccepted])
@@ -739,7 +743,7 @@ bool File_Mpeg4v::Header_Parser_Fill_Size()
     //Must wait more data?
     if (Buffer_Offset_Temp+4>Buffer_Size)
     {
-        if (FrameIsAlwaysComplete || File_Offset+Buffer_Size==File_Size)
+        if (FrameIsAlwaysComplete || Config->IsFinishing)
             Buffer_Offset_Temp=Buffer_Size; //We are sure that the next bytes are a start
         else
             return false;
